@@ -24,13 +24,11 @@
 #define _GNU_SOURCE
 #endif
 
-#include <errno.h>
 #include <fcntl.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <sys/mman.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -39,15 +37,6 @@
 
 #include <libtracker-extract/tracker-extract.h>
 
-static void extract_abw (const gchar          *uri,
-                         TrackerSparqlBuilder *preupdate,
-                         TrackerSparqlBuilder *metadata);
-
-static TrackerExtractData data[] = {
-	{ "application/x-abiword", extract_abw },
-	{ NULL, NULL }
-};
-
 typedef struct AbwParserData AbwParserData;
 typedef enum {
 	ABW_PARSER_TAG_UNHANDLED,
@@ -172,67 +161,40 @@ static GMarkupParser parser = {
 	NULL, NULL
 };
 
-static void
-extract_abw (const gchar          *uri,
-             TrackerSparqlBuilder *preupdate,
-             TrackerSparqlBuilder *metadata)
+G_MODULE_EXPORT gboolean
+tracker_extract_get_metadata (const gchar          *uri,
+                              const gchar          *mimetype,
+                              TrackerSparqlBuilder *preupdate,
+                              TrackerSparqlBuilder *metadata,
+                              GString              *where)
 {
-	int fd;
+	GMappedFile *file;
 	gchar *filename, *contents;
 	GError *error = NULL;
+	gboolean retval = FALSE;
 	gsize len;
-	struct stat st;
 
 	filename = g_filename_from_uri (uri, NULL, &error);
 
 	if (error) {
 		g_warning ("Could not get filename: %s\n", error->message);
 		g_error_free (error);
-		return;
-	}
-
-	fd = g_open (filename, O_RDONLY | O_NOATIME, 0);
-	if (fd == -1 && errno == EPERM) {
-		fd = g_open (filename, O_RDONLY, 0);
-	}
-
-	if (fd == -1) {
-		g_warning ("Could not open abw file '%s': %s\n",
-		           filename,
-		           g_strerror (errno));
-		g_free (filename);
-		return;
-	}
-
-	if (fstat (fd, &st) == -1) {
-		g_warning ("Could not fstat abw file '%s': %s\n",
-		           filename,
-		           g_strerror (errno));
-		close (fd);
-		g_free (filename);
-		return;
-	}
-
-	if (st.st_size == 0) {
-		contents = NULL;
-		len = 0;
-	} else {
-		contents = (gchar *) mmap (NULL, st.st_size, PROT_READ, MAP_PRIVATE, fd, 0);
-		if (contents == NULL) {
-			g_warning ("Could not mmap abw file '%s': %s\n",
-			           filename,
-			           g_strerror (errno));
-			close (fd);
-			g_free (filename);
-			return;
-		}
-		len = st.st_size;
+		return retval;
 	}
 
+	file = g_mapped_file_new (filename, FALSE, &error);
 	g_free (filename);
 
+	if (error) {
+		g_warning ("Could not mmap abw file: %s\n", error->message);
+		g_error_free (error);
+		return retval;
+	}
+
+	contents = g_mapped_file_get_contents (file);
+	len = g_mapped_file_get_length (file);
+
 	if (contents) {
-		GError *error = NULL;
 		GMarkupParseContext *context;
 		AbwParserData data = { 0 };
 
@@ -255,20 +217,13 @@ extract_abw (const gchar          *uri,
 				g_string_free (data.content, TRUE);
 			}
 
+			retval = TRUE;
 		}
 
 		g_markup_parse_context_free (context);
 	}
 
-	if (contents) {
-		munmap (contents, len);
-	}
+	g_mapped_file_unref (file);
 
-	close (fd);
-}
-
-TrackerExtractData *
-tracker_extract_get_data (void)
-{
-	return data;
+	return retval;
 }
