@@ -22,15 +22,6 @@
 #include <libtracker-common/tracker-file-utils.h>
 #include <libtracker-extract/tracker-extract.h>
 
-static void extract_icon (const gchar          *filename,
-                          TrackerSparqlBuilder *preupdate,
-                          TrackerSparqlBuilder *metadata);
-
-static TrackerExtractData data[] = {
-	{ "image/vnd.microsoft.icon", extract_icon },
-	{ NULL, NULL }
-};
-
 #define ICON_HEADER_SIZE_16 3
 #define ICON_IMAGE_METADATA_SIZE_8 16
 
@@ -133,10 +124,12 @@ find_max_width_and_height (const gchar *uri,
 	return TRUE;
 }
 
-static void
-extract_icon (const gchar          *uri,
-              TrackerSparqlBuilder *preupdate,
-              TrackerSparqlBuilder *metadata)
+G_MODULE_EXPORT gboolean
+tracker_extract_get_metadata (const gchar          *uri,
+                              const gchar          *mimetype,
+                              TrackerSparqlBuilder *preupdate,
+                              TrackerSparqlBuilder *metadata,
+                              GString              *where)
 {
 	guint max_width;
 	guint max_height;
@@ -158,10 +151,6 @@ extract_icon (const gchar          *uri,
 			tracker_sparql_builder_object_int64 (metadata, (gint64) max_height);
 		}
 	}
-}
 
-TrackerExtractData *
-tracker_extract_get_data (void)
-{
-	return data;
+	return TRUE;
 }
