@@ -1004,8 +1004,16 @@ id3v2_strlen (const gchar  encoding,
 	switch (encoding) {
 	case 0x01:
 	case 0x02:
+		
 		/* UTF-16, string terminated by two NUL bytes */
-		pos = memmem (text, len, "\0\0", 2);
+		pos = memmem (text, len, "\0\0\0", 3);
+
+		if (pos == NULL) {
+			pos = memmem (text, len, "\0\0", 2);
+		} else {
+			pos++;
+		}
+
 		if (pos != NULL) {
 			return pos - text;
 		} else {
@@ -1019,7 +1027,8 @@ id3v2_strlen (const gchar  encoding,
 static gchar *
 id3v24_text_to_utf8 (const gchar  encoding,
                      const gchar *text,
-                     gssize       len)
+                     gssize       len,
+                     id3tag      *info)
 {
 	/* This byte describes the encoding
 	 * try to convert strings to UTF-8
@@ -1035,7 +1044,7 @@ id3v24_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	case 0x01 :
 		return convert_to_encoding (text,
@@ -1060,7 +1069,7 @@ id3v24_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	}
 }
@@ -1068,7 +1077,8 @@ id3v24_text_to_utf8 (const gchar  encoding,
 static gchar *
 id3v2_text_to_utf8 (const gchar  encoding,
                     const gchar *text,
-                    gssize       len)
+                    gssize       len,
+                    id3tag      *info)
 {
 	/* This byte describes the encoding
 	 * try to convert strings to UTF-8
@@ -1084,7 +1094,7 @@ id3v2_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	case 0x01 :
 		/* return g_convert (text, */
@@ -1102,7 +1112,7 @@ id3v2_text_to_utf8 (const gchar  encoding,
 		return convert_to_encoding (text,
 		                            len,
 		                            "UTF-8",
-		                            "Windows-1252",
+		                            info->encoding ? info->encoding : "Windows-1252",
 		                            NULL, NULL, NULL);
 	}
 }
@@ -1220,7 +1230,7 @@ get_id3v24_tags (id3v24frame           frame,
 		offset        = 4 + text_desc_len + id3v2_nul_size (text_encode);
 		text          = &data[pos + offset]; /* <full text string according to encoding> */
 
-		word = id3v24_text_to_utf8 (text_encode, text, csize - offset);
+		word = id3v24_text_to_utf8 (text_encode, text, csize - offset, info);
 
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
@@ -1236,7 +1246,7 @@ get_id3v24_tags (id3v24frame           frame,
 		gchar *word;
 
 		/* text frames */
-		word = id3v24_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		word = id3v24_text_to_utf8 (data[pos], &data[pos + 1], csize - 1, info);
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
 		}
@@ -1406,7 +1416,7 @@ get_id3v23_tags (id3v24frame           frame,
 		offset        = 4 + text_desc_len + id3v2_nul_size (text_encode);
 		text          = &data[pos + offset]; /* <full text string according to encoding> */
 
-		word = id3v2_text_to_utf8 (text_encode, text, csize - offset);
+		word = id3v2_text_to_utf8 (text_encode, text, csize - offset, info);
 
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
@@ -1423,7 +1433,7 @@ get_id3v23_tags (id3v24frame           frame,
 		gchar *word;
 
 		/* text frames */
-		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1, info);
 
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
@@ -1569,7 +1579,7 @@ get_id3v20_tags (id3v2frame            frame,
 		/* text frames */
 		gchar *word;
 
-		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1);
+		word = id3v2_text_to_utf8 (data[pos], &data[pos + 1], csize - 1, info);
 		if (!tracker_is_empty_string (word)) {
 			g_strstrip (word);
 		}
