@@ -1,18 +1,18 @@
 /*
  * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
  */
@@ -68,16 +68,6 @@ typedef struct {
 	gchar *publisher;
 } VorbisData;
 
-static void extract_vorbis (const char            *uri,
-                            TrackerSparqlBuilder  *preupdate,
-                            TrackerSparqlBuilder  *metadata);
-
-static TrackerExtractData extract_data[] = {
-	{ "audio/x-vorbis+ogg", extract_vorbis },
-	{ "application/ogg", extract_vorbis },
-	{ NULL, NULL }
-};
-
 static gchar *
 ogg_get_comment (vorbis_comment *vc,
                  const gchar    *label)
@@ -95,10 +85,12 @@ ogg_get_comment (vorbis_comment *vc,
 	}
 }
 
-static void
-extract_vorbis (const char *uri,
-                TrackerSparqlBuilder *preupdate,
-                TrackerSparqlBuilder *metadata)
+G_MODULE_EXPORT gboolean
+tracker_extract_get_metadata (const char           *uri,
+                              const gchar          *mimetype,
+                              TrackerSparqlBuilder *preupdate,
+                              TrackerSparqlBuilder *metadata,
+                              GString              *where)
 {
 	VorbisData vd = { 0 };
 	MergeData md = { 0 };
@@ -115,12 +107,12 @@ extract_vorbis (const char *uri,
 	g_free (filename);
 
 	if (!f) {
-		return;
+		return FALSE;
 	}
 
 	if (ov_open (f, &vf, NULL, 0) < 0) {
 		tracker_file_close (f, FALSE);
-		return;
+		return FALSE;
 	}
 
 	tracker_sparql_builder_predicate (metadata, "a");
@@ -465,10 +457,6 @@ extract_vorbis (const char *uri,
 
 	/* NOTE: This calls fclose on the file */
 	ov_clear (&vf);
-}
 
-TrackerExtractData *
-tracker_extract_get_data (void)
-{
-	return extract_data;
+	return TRUE;
 }
