@@ -45,32 +45,6 @@
 #warning Main thread traces enabled
 #endif /* THREAD_ENABLE_TRACE */
 
-#define MAX_EXTRACT_TIME 10
-
-#define UNKNOWN_METHOD_MESSAGE "Method \"%s\" with signature \"%s\" on " \
-                               "interface \"%s\" doesn't exist, expected \"%s\""
-
-static const gchar introspection_xml[] =
-  "<node>"
-  "  <interface name='org.freedesktop.Tracker1.Extract'>"
-  "    <method name='GetPid'>"
-  "      <arg type='i' name='value' direction='out' />"
-  "    </method>"
-  "    <method name='GetMetadata'>"
-  "      <arg type='s' name='uri' direction='in' />"
-  "      <arg type='s' name='mime' direction='in' />"
-  "      <arg type='s' name='preupdate' direction='out' />"
-  "      <arg type='s' name='embedded' direction='out' />"
-  "      <arg type='s' name='where' direction='out' />"
-  "    </method>"
-  "    <method name='GetMetadataFast'>"
-  "      <arg type='s' name='uri' direction='in' />"
-  "      <arg type='s' name='mime' direction='in' />"
-  "      <arg type='h' name='fd' direction='in' />"
-  "    </method>"
-  "  </interface>"
-  "</node>";
-
 #define TRACKER_EXTRACT_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), TRACKER_TYPE_EXTRACT, TrackerExtractPrivate))
 
 extern gboolean debug;
@@ -82,6 +56,20 @@ typedef struct {
 
 typedef struct {
 	GHashTable *statistics_data;
+	GList *running_tasks;
+
+	/* used to maintain the running tasks
+	 * and stats from different threads
+	 */
+	GMutex *task_mutex;
+
+	/* Thread pool for multi-threaded extractors */
+	GThreadPool *thread_pool;
+
+	/* module -> async queue hashtable
+	 * for single-threaded extractors
+	 */
+	GHashTable *single_thread_extractors;
 
 	gboolean disable_shutdown;
 	gboolean force_internal_extractors;
@@ -96,10 +84,22 @@ typedef struct {
 	GAsyncResult *res;
 	gchar *file;
 	gchar *mimetype;
+
+	TrackerMimetypeInfo *mimetype_handlers;
+
+	/* to be fed from mimetype_handlers */
+	TrackerExtractMetadataFunc cur_func;
+	GModule *cur_module;
+
+	guint signal_id;
+	guint success : 1;
 } TrackerExtractTask;
 
 static void tracker_extract_finalize (GObject *object);
 static void report_statistics        (GObject *object);
+static gboolean get_metadata         (TrackerExtractTask *task);
+static gboolean dispatch_task_cb     (TrackerExtractTask *task);
+
 
 G_DEFINE_TYPE(TrackerExtract, tracker_extract, G_TYPE_OBJECT)
 
@@ -132,7 +132,12 @@ tracker_extract_init (TrackerExtract *object)
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 	priv->statistics_data = g_hash_table_new_full (NULL, NULL, NULL,
-						       (GDestroyNotify) statistics_data_free);
+	                                               (GDestroyNotify) statistics_data_free);
+	priv->single_thread_extractors = g_hash_table_new (NULL, NULL);
+	priv->thread_pool = g_thread_pool_new ((GFunc) get_metadata,
+	                                       NULL, 10, TRUE, NULL);
+
+	priv->task_mutex = g_mutex_new ();
 }
 
 static void
@@ -142,6 +147,11 @@ tracker_extract_finalize (GObject *object)
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 
+	/* FIXME: Shutdown modules? */
+
+	g_hash_table_destroy (priv->single_thread_extractors);
+	g_thread_pool_free (priv->thread_pool, TRUE, FALSE);
+
 	if (!priv->disable_summary_on_finalize) {
 		report_statistics (object);
 	}
@@ -164,6 +174,8 @@ report_statistics (GObject *object)
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 
+	g_mutex_lock (priv->task_mutex);
+
 	g_message ("--------------------------------------------------");
 	g_message ("Statistics:");
 
@@ -194,6 +206,8 @@ report_statistics (GObject *object)
 	}
 
 	g_message ("--------------------------------------------------");
+
+	g_mutex_unlock (priv->task_mutex);
 }
 
 TrackerExtract *
@@ -219,27 +233,59 @@ tracker_extract_new (gboolean     disable_shutdown,
 	return object;
 }
 
+static void
+notify_task_finish (TrackerExtractTask *task,
+                    gboolean            success)
+{
+	TrackerExtract *extract;
+	TrackerExtractPrivate *priv;
+	StatisticsData *stats_data;
+
+	extract = task->extract;
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	/* Reports and ongoing tasks may be
+	 * accessed from other threads.
+	 */
+	g_mutex_lock (priv->task_mutex);
+
+	stats_data = g_hash_table_lookup (priv->statistics_data,
+	                                  task->cur_module);
+
+	if (!stats_data) {
+		stats_data = g_slice_new0 (StatisticsData);
+		g_hash_table_insert (priv->statistics_data,
+		                     task->cur_module,
+		                     stats_data);
+	}
+
+	stats_data->extracted_count++;
+
+	if (!success) {
+		stats_data->failed_count++;
+	}
+
+	priv->running_tasks = g_list_remove (priv->running_tasks, task);
+
+	g_mutex_unlock (priv->task_mutex);
+}
+
 static gboolean
-get_file_metadata (TrackerExtract         *extract,
-                   const gchar            *uri,
-                   const gchar            *mime,
+get_file_metadata (TrackerExtractTask     *task,
                    TrackerSparqlBuilder  **preupdate_out,
                    TrackerSparqlBuilder  **statements_out,
                    gchar                 **where_out)
 {
-	TrackerExtractPrivate *priv;
 	TrackerSparqlBuilder *statements, *preupdate;
 	GString *where;
 	gchar *mime_used = NULL;
 #ifdef HAVE_LIBSTREAMANALYZER
 	gchar *content_type = NULL;
 #endif
-	gint items;
+	gint items = 0;
 
 	g_debug ("Extracting...");
 
-	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
-
 	*preupdate_out = NULL;
 	*statements_out = NULL;
 	*where_out = NULL;
@@ -269,10 +315,9 @@ get_file_metadata (TrackerExtract         *extract,
 	}
 #endif /* HAVE_LIBSTREAMANALYZER */
 
-	if (mime && *mime) {
+	if (task->mimetype && *task->mimetype) {
 		/* We know the mime */
-		mime_used = g_strdup (mime);
-		g_strstrip (mime_used);
+		mime_used = g_strdup (task->mimetype);
 	}
 #ifdef HAVE_LIBSTREAMANALYZER
 	else if (content_type && *content_type) {
@@ -282,165 +327,145 @@ get_file_metadata (TrackerExtract         *extract,
 	}
 #endif /* HAVE_LIBSTREAMANALYZER */
 	else {
-		GFile *file;
-		GFileInfo *info;
-		GError *error = NULL;
-
-		file = g_file_new_for_uri (uri);
-		if (!file) {
-			g_warning ("Could not create GFile for uri:'%s'",
-			           uri);
-			g_object_unref (statements);
-			g_object_unref (preupdate);
-			g_string_free (where, TRUE);
-			return FALSE;
-		}
-
-		info = g_file_query_info (file,
-		                          G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-		                          G_FILE_QUERY_INFO_NONE,
-		                          NULL,
-		                          &error);
-
-		if (error || !info) {
-			/* FIXME: Propagate error */
-			g_error_free (error);
-
-			if (info) {
-				g_object_unref (info);
-			}
-
-			g_object_unref (file);
-			g_object_unref (statements);
-			g_object_unref (preupdate);
-			g_string_free (where, TRUE);
-
-			return FALSE;
-		}
-
-		mime_used = g_strdup (g_file_info_get_content_type (info));
-
-		g_object_unref (info);
-		g_object_unref (file);
+		return FALSE;
 	}
 
 	/* Now we have sanity checked everything, actually get the
 	 * data we need from the extractors.
 	 */
 	if (mime_used) {
-		TrackerExtractMetadataFunc func;
-		GModule *module;
+		if (task->cur_func) {
+			g_debug ("  Using %s...", g_module_name (task->cur_module));
 
-		module = tracker_extract_module_manager_get_for_mimetype (mime_used, &func);
-
-		if (module) {
-			StatisticsData *data;
-
-			g_debug ("  Using %s...", g_module_name (module));
-
-			(func) (uri, mime_used, preupdate, statements, where);
+			(task->cur_func) (task->file, mime_used, preupdate, statements, where);
 
 			items = tracker_sparql_builder_get_length (statements);
 
-			data = g_hash_table_lookup (priv->statistics_data, module);
-
-			if (!data) {
-				data = g_slice_new0 (StatisticsData);
-				g_hash_table_insert (priv->statistics_data, module, data);
-			}
-
-			data->extracted_count++;
-
 			if (items > 0) {
 				tracker_sparql_builder_insert_close (statements);
 
-				*preupdate_out = preupdate;
-				*statements_out = statements;
-				*where_out = g_string_free (where, FALSE);
-
 				g_debug ("Done (%d items)", items);
 
-				g_free (mime_used);
-				return TRUE;
-			} else {
-				data->failed_count++;
+				task->success = TRUE;
 			}
-		} else {
-			g_debug ("  No extractor was available for this mime type:'%s'",
-			         mime_used);
-
-			priv->unhandled_count++;
 		}
 
 		g_free (mime_used);
 	}
 
-	items = tracker_sparql_builder_get_length (statements);
-
-	if (items > 0) {
-		tracker_sparql_builder_insert_close (statements);
-	}
-
 	*preupdate_out = preupdate;
 	*statements_out = statements;
 	*where_out = g_string_free (where, FALSE);
 
-	g_debug ("No extractor or failed (%d items)", items);
+	if (items == 0) {
+		g_debug ("No extractor or failed");
+	}
 
-	return TRUE;
+	return (items > 0);
 }
 
+/* This function is called on the thread calling g_cancellable_cancel() */
 static void
-tracker_extract_info_free (TrackerExtractInfo *info)
+task_cancellable_cancelled_cb (GCancellable       *cancellable,
+                               TrackerExtractTask *task)
 {
-	if (info->statements) {
-		g_object_unref (info->statements);
+	TrackerExtractPrivate *priv;
+	TrackerExtract *extract;
+
+	extract = task->extract;
+	priv = TRACKER_EXTRACT_GET_PRIVATE (extract);
+
+	g_mutex_lock (priv->task_mutex);
+
+	if (g_list_find (priv->running_tasks, task)) {
+		g_message ("Cancelled task for '%s' was currently being "
+		           "processed, _exit()ing immediately",
+		           task->file);
+		_exit (0);
 	}
 
-	if (info->preupdate) {
-		g_object_unref (info->preupdate);
-	}
-
-	g_free (info->where);
-	g_slice_free (TrackerExtractInfo, info);
+	g_mutex_unlock (priv->task_mutex);
 }
 
 static TrackerExtractTask *
 extract_task_new (TrackerExtract *extract,
-                  const gchar    *file,
+                  const gchar    *uri,
                   const gchar    *mimetype,
                   GCancellable   *cancellable,
                   GAsyncResult   *res)
 {
 	TrackerExtractTask *task;
 
+	if (!mimetype) {
+		GFile *file;
+		GFileInfo *info;
+
+		file = g_file_new_for_uri (uri);
+		info = g_file_query_info (file,
+		                          G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+		                          G_FILE_QUERY_INFO_NONE,
+		                          NULL, NULL);
+
+		if (info) {
+			mimetype = g_strdup (g_file_info_get_content_type (info));
+			g_object_unref (info);
+		} else {
+			g_warning ("Could not get mimetype for '%s'", uri);
+			return NULL;
+		}
+
+		g_object_unref (file);
+	}
+
 	task = g_slice_new0 (TrackerExtractTask);
-	task->cancellable = cancellable;
-	task->res = g_object_ref (res);
-	task->file = g_strdup (file);
+	task->cancellable = (cancellable) ? g_object_ref (cancellable) : NULL;
+	task->res = (res) ? g_object_ref (res) : NULL;
+	task->file = g_strdup (uri);
 	task->mimetype = g_strdup (mimetype);
 	task->extract = extract;
 
+	if (task->cancellable) {
+		task->signal_id = g_cancellable_connect (cancellable,
+		                                         G_CALLBACK (task_cancellable_cancelled_cb),
+		                                         task, NULL);
+	}
+
 	return task;
 }
 
 static void
 extract_task_free (TrackerExtractTask *task)
 {
-	g_object_unref (task->res);
+	if (task->cancellable && task->signal_id != 0) {
+		g_cancellable_disconnect (task->cancellable, task->signal_id);
+	}
+
+	notify_task_finish (task, task->success);
+
+	if (task->res) {
+		g_object_unref (task->res);
+	}
+
+	if (task->cancellable) {
+		g_object_unref (task->cancellable);
+	}
+
 	g_free (task->file);
 	g_free (task->mimetype);
 	g_slice_free (TrackerExtractTask, task);
 }
 
 static gboolean
-get_metadata_cb (gpointer user_data)
+get_metadata (TrackerExtractTask *task)
 {
-	TrackerExtractTask *task = user_data;
 	TrackerExtractInfo *info;
+	TrackerSparqlBuilder *preupdate, *statements;
+	gchar *where = NULL;
+
+	preupdate = statements = NULL;
 
 #ifdef THREAD_ENABLE_TRACE
-	g_debug ("Thread:%p (Main) --> File:'%s' - Extracted",
+	g_debug ("Thread:%p --> File:'%s' - Extracted",
 	         g_thread_self (),
 	         task->file);
 #endif /* THREAD_ENABLE_TRACE */
@@ -451,30 +476,187 @@ get_metadata_cb (gpointer user_data)
 		                                 TRACKER_DBUS_ERROR, 0,
 		                                 "Extraction of '%s' was cancelled",
 		                                 task->file);
+
 		extract_task_free (task);
 		return FALSE;
 	}
 
-	info = g_slice_new (TrackerExtractInfo);
+	if (get_file_metadata (task, &preupdate, &statements, &where)) {
+		info = tracker_extract_info_new ((preupdate) ? tracker_sparql_builder_get_result (preupdate) : NULL,
+		                                 (statements) ? tracker_sparql_builder_get_result (statements) : NULL,
+		                                 where);
 
-	if (get_file_metadata (task->extract,
-	                       task->file, task->mimetype,
-	                       &info->preupdate,
-	                       &info->statements,
-	                       &info->where)) {
 		g_simple_async_result_set_op_res_gpointer ((GSimpleAsyncResult *) task->res,
 		                                           info,
 		                                           (GDestroyNotify) tracker_extract_info_free);
+
+		g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
+		extract_task_free (task);
 	} else {
-		g_simple_async_result_set_error ((GSimpleAsyncResult *) task->res,
-		                                 TRACKER_DBUS_ERROR, 0,
-		                                 "Could not get any metadata for uri:'%s' and mime:'%s'",
-		                                 task->file, task->mimetype);
-		tracker_extract_info_free (info);
+		if (preupdate) {
+			g_object_unref (preupdate);
+		}
+
+		if (statements) {
+			g_object_unref (statements);
+		}
+
+		g_free (where);
+
+		/* Reinject the task into the main thread
+		 * queue, so the next module kicks in.
+		 */
+		g_idle_add ((GSourceFunc) dispatch_task_cb, task);
 	}
 
-	g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
-	extract_task_free (task);
+	return FALSE;
+}
+
+static void
+single_thread_get_metadata (GAsyncQueue *queue)
+{
+	while (TRUE) {
+		TrackerExtractTask *task;
+
+		task = g_async_queue_pop (queue);
+		g_message ("Dispatching '%s' in dedicated thread", task->file);
+		get_metadata (task);
+	}
+}
+
+/* This function is executed in the main thread, decides the
+ * module that's going to be run for a given task, and dispatches
+ * the task according to the threading strategy of that module.
+ */
+static gboolean
+dispatch_task_cb (TrackerExtractTask *task)
+{
+	TrackerModuleThreadAwareness thread_awareness;
+	TrackerExtractPrivate *priv;
+	GError *error = NULL;
+	GModule *module;
+
+#ifdef THREAD_ENABLE_TRACE
+	g_debug ("Thread:%p (Main) <-- File:'%s' - Dispatching\n",
+	         g_thread_self (),
+	         task->file);
+#endif /* THREAD_ENABLE_TRACE */
+
+	priv = TRACKER_EXTRACT_GET_PRIVATE (task->extract);
+
+	if (!task->mimetype) {
+		error = g_error_new (TRACKER_DBUS_ERROR, 0,
+		                     "No mimetype for '%s'",
+		                     task->file);
+	} else {
+		if (!task->mimetype_handlers) {
+			/* First iteration for task, get the mimetype handlers */
+			task->mimetype_handlers = tracker_extract_module_manager_get_mimetype_handlers (task->mimetype);
+
+			if (!task->mimetype_handlers) {
+				error = g_error_new (TRACKER_DBUS_ERROR, 0,
+				                     "No mimetype extractor handlers for uri:'%s' and mime:'%s'",
+				                     task->file, task->mimetype);
+			}
+		} else {
+			/* Any further iteration, should happen rarely if
+			 * most specific handlers know nothing about the file
+			 */
+			g_message ("Trying next extractor for '%s'", task->file);
+
+			if (!tracker_mimetype_info_iter_next (task->mimetype_handlers)) {
+				g_message ("  There's no next extractor");
+
+				error = g_error_new (TRACKER_DBUS_ERROR, 0,
+				                     "Could not get any metadata for uri:'%s' and mime:'%s'",
+				                     task->file, task->mimetype);
+			}
+		}
+	}
+
+	if (error) {
+		g_simple_async_result_set_from_error ((GSimpleAsyncResult *) task->res, error);
+		g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
+		extract_task_free (task);
+		g_error_free (error);
+
+		return FALSE;
+	}
+
+	task->cur_module = module = tracker_mimetype_info_get_module (task->mimetype_handlers, &task->cur_func, &thread_awareness);
+
+	if (!module || !task->cur_func) {
+		g_warning ("Discarding task with no module '%s'", task->file);
+		priv->unhandled_count++;
+		return FALSE;
+	}
+
+	g_mutex_lock (priv->task_mutex);
+	priv->running_tasks = g_list_prepend (priv->running_tasks, task);
+	g_mutex_unlock (priv->task_mutex);
+
+	switch (thread_awareness) {
+	case TRACKER_MODULE_NONE:
+		/* Error out */
+		g_simple_async_result_set_error ((GSimpleAsyncResult *) task->res,
+		                                 TRACKER_DBUS_ERROR, 0,
+		                                 "Module '%s' initialization failed",
+		                                 g_module_name (module));
+		g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
+		extract_task_free (task);
+		break;
+	case TRACKER_MODULE_MAIN_THREAD:
+		/* Dispatch the task right away in this thread */
+		g_message ("Dispatching '%s' in main thread", task->file);
+		get_metadata (task);
+		break;
+	case TRACKER_MODULE_SINGLE_THREAD:
+	{
+		GAsyncQueue *async_queue;
+
+		async_queue = g_hash_table_lookup (priv->single_thread_extractors, module);
+
+		if (!async_queue) {
+			/* No thread created yet for this module, create it
+			 * together with the async queue used to pass data to it
+			 */
+			async_queue = g_async_queue_new ();
+
+			g_thread_create ((GThreadFunc) single_thread_get_metadata,
+			                 g_async_queue_ref (async_queue),
+			                 FALSE, &error);
+
+			if (error) {
+				g_simple_async_result_set_from_error ((GSimpleAsyncResult *) task->res, error);
+				g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
+				extract_task_free (task);
+				g_error_free (error);
+
+				return FALSE;
+			}
+
+			g_hash_table_insert (priv->single_thread_extractors, module, async_queue);
+		}
+
+		g_async_queue_push (async_queue, task);
+	}
+		break;
+	case TRACKER_MODULE_MULTI_THREAD:
+		/* Put task in thread pool */
+		g_message ("Dispatching '%s' in thread pool", task->file);
+		g_thread_pool_push (priv->thread_pool, task, &error);
+
+		if (error) {
+			g_simple_async_result_set_from_error ((GSimpleAsyncResult *) task->res, error);
+			g_simple_async_result_complete_in_idle ((GSimpleAsyncResult *) task->res);
+			extract_task_free (task);
+			g_error_free (error);
+
+			return FALSE;
+		}
+
+		break;
+	}
 
 	return FALSE;
 }
@@ -496,7 +678,7 @@ tracker_extract_file (TrackerExtract      *extract,
 	g_return_if_fail (cb != NULL);
 
 #ifdef THREAD_ENABLE_TRACE
-	g_debug ("Thread:%p (Main) <-- File:'%s' - Extracting\n",
+	g_debug ("Thread:%p <-- File:'%s' - Extracting\n",
 	         g_thread_self (),
 	         file);
 #endif /* THREAD_ENABLE_TRACE */
@@ -504,10 +686,13 @@ tracker_extract_file (TrackerExtract      *extract,
 	res = g_simple_async_result_new (G_OBJECT (extract), cb, user_data, NULL);
 
 	task = extract_task_new (extract, file, mimetype, cancellable, G_ASYNC_RESULT (res));
-	g_idle_add (get_metadata_cb, task);
 
-	/* task takes a ref */
-	g_object_unref (res);
+	if (task) {
+		g_idle_add ((GSourceFunc) dispatch_task_cb, task);
+
+		/* task takes a ref */
+		g_object_unref (res);
+	}
 }
 
 void
@@ -518,13 +703,32 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
 	TrackerSparqlBuilder *statements, *preupdate;
 	gchar *where;
 	TrackerExtractPrivate *priv;
+	TrackerExtractTask *task;
+	TrackerExtractInitFunc init_func;
 
 	priv = TRACKER_EXTRACT_GET_PRIVATE (object);
 	priv->disable_summary_on_finalize = TRUE;
 
 	g_return_if_fail (uri != NULL);
 
-	if (get_file_metadata (object, uri, mime, &preupdate, &statements, &where)) {
+	g_message ("Extracting...");
+
+	task = extract_task_new (object, uri, mime, NULL, NULL);
+
+	if (!task) {
+		return;
+	}
+
+	task->cur_module = tracker_extract_module_manager_get_for_mimetype (task->mimetype, &init_func, NULL, &task->cur_func);
+
+	if (init_func) {
+		TrackerModuleThreadAwareness ignore;
+
+		/* Initialize module for this single run */
+		(init_func) (&ignore, NULL);
+	}
+
+	if (get_file_metadata (task, &preupdate, &statements, &where)) {
 		const gchar *preupdate_str, *statements_str;
 
 		preupdate_str = statements_str = NULL;
@@ -548,4 +752,6 @@ tracker_extract_get_metadata_by_cmdline (TrackerExtract *object,
 		g_object_unref (preupdate);
 		g_free (where);
 	}
+
+	extract_task_free (task);
 }
